library simple_connect;

import 'package:connectivity/connectivity.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:http/http.dart' as http;
import 'dart:convert';
import 'dart:io';

import 'package:progress_dialog/progress_dialog.dart';

enum requestType {
  get,
  post,
  put,
  delete
}

class CleanConnect {
  static String _mainUrl = '';
  static Map<String, String> _header = {"Content-Type": "application/json"};
  static List<int> _statusSuccess = [200, 201, 202];
  static ProgressDialog _progressDialog;

  static initProgressDialog(BuildContext context, {String message = 'Loading...'}){
    _progressDialog = new ProgressDialog(context, ProgressDialogType.Normal);
    _progressDialog.setMessage(message);
  }

  static bool setProgressDialogMessage(String message) {
    if (_progressDialog != null){
      _progressDialog.setMessage(message);
      return true;
    }return false;
  }

  static _setProgressStatus(bool useProgress, bool status){
    if(_progressDialog != null && useProgress){
      if(status)
        _progressDialog.show();
      else
        _progressDialog.hide();
    }
  }

  static bool getProgressDialogStatus(){
    if(_progressDialog != null){
      return _progressDialog.isShowing();
    }return false;
  }

  static setHeader({
                    String contentType = "application/json",
                    String authorization = "bearer"
                  }){
    _header = {"Content-Type": contentType,
                HttpHeaders.authorizationHeader: authorization};
  }

  static setMainUrl(String _url){
    _mainUrl = _url;
  }

  static Future<bool> _errorDialog(context) async{
    await showDialog(
        context: context,
        barrierDismissible: false,
        builder: (context) => AlertDialog(
          title: Text('Connection'),
          content: Text('No Internet Access'),
          actions: <Widget>[
            FlatButton(
              child: Text('Try again'),
              onPressed: () {
                Navigator.pop(context);
              },
            )
          ],
        )
    );
    return true;
  }

  static Future<Rest> cRequest(requestType _type,
                              String _route,
                              BuildContext context,
                              {
                                dynamic sBody,
                                String otherMainUrl,
                                bool useProgress = false,
                                bool useDialogs = true,
                                bool jsonDecode = true
                              }) async{
    _setProgressStatus(useProgress, true);
    if(await _checkConn()){
      http.Response response = await _getRequested(otherMainUrl ?? _mainUrl, _route, _type, sBody);
      if(_statusSuccess.contains(response.statusCode)){
        return Rest(true, response.statusCode,
            jsonDecode ? json.decode(response.body) : response.body);
      }

      return Rest(false, response.statusCode,
          jsonDecode ? json.decode(response.body) : response.body);

    }else{
      if(useDialogs) {
        _setProgressStatus(useProgress, false);
        await _errorDialog(context);
        Rest r = Rest(false, 0, {});

        while(r.statusCode == 0){
          r = await cRequest(_type, _route, context,
              sBody: sBody,
              otherMainUrl: otherMainUrl,
              useProgress: useProgress,
              useDialogs: false,
              jsonDecode: jsonDecode);
        }
      }
      return Rest(false, 0, {});
    }
  }

  static Future<http.Response> _getRequested(_mainUrl, _route, _type, _sBody) async {
    switch (_type) {
      case requestType.get:
        return await http.get('$_mainUrl$_route', headers: _header);

      case requestType.delete:
        return await http.delete('$_mainUrl$_route', headers: _header);

      case requestType.post:
        return await http.post('$_mainUrl$_route', headers: _header, body: json.encode(_sBody));

      case requestType.put:
        return await http.put('$_mainUrl$_route', headers: _header, body: json.encode(_sBody));

      default: return null;
    }
  }

  static Future<bool> _checkConn() async {
    var connectivityResult = await (Connectivity().checkConnectivity());
    if (connectivityResult == ConnectivityResult.mobile) {
      return true;
    } else if (connectivityResult == ConnectivityResult.wifi) {
      return true;
    }
    return false;
  }
}

class Rest{
  final bool success;
  final dynamic statusCode;
  final dynamic data;

  Rest(this.success, this.statusCode, this.data);

  factory Rest.fromJson(body){
    return Rest(
        body['success'] ?? false,
        body['error'] ?? 'Error in body',
        body['data'] ?? body
    );
  }

  @override
  String toString() {
    return {'success': success, 'code': statusCode, 'data': data}.toString();
  }
}